/// <reference types="./global.d.ts" />
/** /Context/Query
 *
 * @author Alex Malotky
 */
import { Object as ObjectValue } from "../Validation/Type";
import DataObject, { TypeOf, DataConstraints, ObjectProperties } from "../Validation";
/** Database Query Builder
 *
 * Currntly only works with Cloudflare D1Database
 */
export default class QueryBuilder {
    #private;
    constructor(env: Env);
    private get db();
    /** Insert Value into Database
     *
     * @param {ObjectValidator} validator
     * @param {Object} value
     */
    insert<P extends ObjectProperties>(object: DataObject<P>, value: ObjectValue<keyof P>): Promise<void>;
    /** Update Value in Database
     *
     * @param {ObjectValidator} validator
     * @param {Object} value
     * @param {ObjectDefaults} constraints
     */
    update<P extends ObjectProperties>(object: DataObject<P>, value: ObjectValue<keyof P>, constraints?: DataConstraints<keyof P>): Promise<void>;
    /**
     *
     */
    delete<P extends ObjectProperties>(object: DataObject<P>, constraints?: DataConstraints<keyof P>): Promise<void>;
    /** Get First Value from Database
     *
     * @param {ObjectValidator} validator
     * @param {ObjectDefaults} constraints
     */
    get<P extends ObjectProperties>(object: DataObject<P>, constraints?: DataConstraints<keyof P>): Promise<TypeOf<DataObject<P>> | null>;
    /** Get ALl Values from Database
     *
     * @param {ObjectValidator} validator
     * @param {ObjectDefaults} constraints
     */
    getAll<P extends ObjectProperties>(object: DataObject<P>, constraints?: DataConstraints<keyof P>): Promise<TypeOf<DataObject<P>>[]>;
}
