"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _QueryBuilder_db;
Object.defineProperty(exports, "__esModule", { value: true });
const Util_1 = require("../Util");
/** Database Query Builder
 *
 * Currntly only works with Cloudflare D1Database
 */
class QueryBuilder {
    constructor(env) {
        _QueryBuilder_db.set(this, void 0);
        for (const name in env) {
            if ((0, Util_1.isDatabaseClass)(env[name])) {
                __classPrivateFieldSet(this, _QueryBuilder_db, env[name], "f");
                break;
            }
        }
    }
    get db() {
        if (__classPrivateFieldGet(this, _QueryBuilder_db, "f") === undefined)
            throw new Error("No database connection!");
        return __classPrivateFieldGet(this, _QueryBuilder_db, "f");
    }
    /** Insert Value into Database
     *
     * @param {ObjectValidator} validator
     * @param {Object} value
     */
    async insert(object, value) {
        const [string, values] = object.buildInsertValues(value);
        await this.db.prepare(`INSERT INTO ${object.name}${string}`)
            .bind(...values).run();
    }
    /** Update Value in Database
     *
     * @param {ObjectValidator} validator
     * @param {Object} value
     * @param {ObjectDefaults} constraints
     */
    async update(object, value, constraints) {
        const [updateString, updateValues] = object.buildUpdateValues(value);
        const [constraintString, constraintValues] = object.buildConstraints(constraints);
        await this.db.prepare(`UPDATE ${object.name} ${updateString} ${constraintString}`)
            .bind(...updateValues.concat(constraintValues)).run();
    }
    /**
     *
     */
    async delete(object, constraints) {
        const [string, values] = object.buildConstraints(constraints);
        await this.db.prepare(`DELETE FROM ${object.name} ${string}`)
            .bind(...values).run();
    }
    /** Get First Value from Database
     *
     * @param {ObjectValidator} validator
     * @param {ObjectDefaults} constraints
     */
    async get(object, constraints) {
        const [string, values] = object.buildConstraints(constraints);
        const result = await this.db.prepare(`SELECT * FROM ${object.name} ${string}`)
            .bind(...values).first();
        if (result === null)
            return null;
        return object.run(result);
    }
    /** Get ALl Values from Database
     *
     * @param {ObjectValidator} validator
     * @param {ObjectDefaults} constraints
     */
    async getAll(object, constraints) {
        const [string, values] = object.buildConstraints(constraints);
        const { results, error } = await this.db.prepare(`SELECT * FROM ${object.name} ${string}`)
            .bind(...values).all();
        if (error)
            throw error;
        return results.map((v) => object.run(v));
    }
}
_QueryBuilder_db = new WeakMap();
exports.default = QueryBuilder;
